/**
 * evt.c - 
 *
 * Copyright (c) 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#include "toolkit.h"
#include "keysyms.h"

/*******************************************************************************
 *
 */

__inline__ int getSourceIdx ( Toolkit* X, Widget* w )
{
  int      n;
  register i;

  if ( w == X->lastWindow ){
	return X->srcIdx;
  }
  else {
	for ( i = (unsigned long) w, n=0; n < X->nWindows; i++, n++ ) {
	  i %= X->nWindows;
	  if ( X->windows[i] == w ){
		X->srcIdx = i;
		X->lastWindow = w;
		return X->srcIdx;
	  }
	  
	  if ( X->windows[i] == NULL )
		return -1;
	}
	return -1;
  }
}

__inline__ int getWidgetIdx ( GtkWidget* gw, GdkEvent* evt )
{
  int      n;
  Widget   *w;
  Widget   *we;

  we = gtk_get_event_widget( evt);
  if ( ( we != NULL) && ( we != gw) )
    return -1;

  for ( n=0; n < X->nWindows;n++ ) {
    w = (Widget*)X->windows[n];
    if ( w && (w->widget == gw) ){
      X->srcIdx = n;
      return X->srcIdx;
    }
  }

  return -1;
}

/* X-to-Java key modifier mapping
 *                                     altGr   : PC
 *     shift       -    ctrl     alt    meta   : Java
 *     Shift    Lock    Ctrl    Mod1    Mod3   : X      symbol
 *  ----------------------------------------------------------
 *         1       1       2       8       4   : Java   value
 *         1       2       4       8      32   : X
 */

__inline__ int keyMod ( int keyState )
{
  int mod = 0;
  if ( keyState & 3 ) mod |= 1; /* shift or lock */
  if ( keyState & 4 ) mod |= 2;
  if ( keyState & 8 ) mod |= 8;
  return mod;
}


gint button_press_callback( GtkWidget* widget, GdkEventButton* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, MouseEvent, queueMouseEvent,
				       X->srcIdx, (X->evtId = MOUSE_PRESSED),
				       evt->button, (int)evt->x, (int)evt->y);
  }
  return ( TRUE);
}

gint button_release_callback( GtkWidget* widget, GdkEventButton* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, MouseEvent, queueMouseEvent,
				       X->srcIdx, (X->evtId = MOUSE_RELEASED),
				       evt->button, (int)evt->x, (int)evt->y);
  }
  return ( TRUE);
}

gint motion_notify_callback( GtkWidget* widget, GdkEventMotion* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, MouseEvent, queueMouseEvent,
				       X->srcIdx, (X->evtId = MOUSE_MOVED),
				       0, (int)evt->x, (int)evt->y);
  }
  return ( TRUE);
}

/*
 * connected to delete_event ( Titlebar closing )
 */
gint delete_callback( GtkWidget* widget, GdkEventAny* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, WindowEvent, queueWindowEvent,
				       X->srcIdx, (X->evtId = WINDOW_CLOSED));
  }
  return ( TRUE);
}

/*
 * connected to destroy signal ( raised by gtk_destroy_widget <- wndDestroyWindow )
 */
gint destroy_callback( GtkWidget* widget, GdkEventAny* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
   (*X->env)->CallStaticObjectMethod( X->env, WindowEvent, queueWindowEvent,
			       X->srcIdx, (X->evtId = WINDOW_CLOSED));
  }
  return ( TRUE);
}

gint expose_callback( GtkWidget* widget, GdkEventExpose* evt, gpointer data) {
  int x = evt->area.x;
  int y = evt->area.y;
  int w = evt->area.width;
  int h = evt->area.height;
  int count = evt->count;
  int xw, yh, exw, eyh;
  GdkEventExpose* ege;

  /*
  while ( count > 0 ) {
    ege = (GdkEventExpose*)gdk_event_get_graphics_expose( widget->window);
    xw = x + w;
    yh = y + h;

    exw = ege->area.x + ege->area.width;
    eyh = ege->area.y + ege->area.height;

    if ( ege->area.x < x ) x = ege->area.x;
    if ( ege->area.y < y ) y = ege->area.y;
    
    w = (exw > xw) ? exw - x : xw - x;
    h = (eyh > yh) ? eyh - y : yh - y;
    evt = ege;
    count--;
  }
  */

  
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, PaintEvent, queuePaintEvent,
				       X->srcIdx, (X->evtId = PAINT),
				       x, y, w, h);
  }

  return ( TRUE);
}

gint key_press_callback( GtkWidget* widget, GdkEventKey* evt, gpointer data) {
  int keysym = evt->keyval;
  int mod     = evt->state;
  int keyCode, keyChar;
 
  if ( (keysym >= 0xff00) || (evt->length == 0) ) {
    keyCode = FKeyCode[keysym & 0xff];
    if ( keyCode < 0 ){
      keyChar = keyCode = -keyCode;
    }
    else {
      keyChar = 0;
    }
  }
  else {
    keyChar = (unsigned char)evt->string[0];
    keyCode = keyChar & ~0x20;
  }

  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, KeyEvent, queueKeyEvent,
				       X->srcIdx, (X->evtId = KEY_PRESSED),
				       keyCode, keyChar, mod);
    (*X->env)->CallStaticObjectMethod( X->env, KeyEvent, queueKeyEvent,
				       X->srcIdx, (X->evtId = KEY_RELEASED),
				       keyCode, keyChar, mod);
  }
  return ( FALSE);
}

gint key_release_callback( GtkWidget* widget, GdkEventKey* evt, gpointer data) {
  return ( TRUE);
}

gint enter_notify_callback( GtkWidget* widget, GdkEventCrossing* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, MouseEvent, queueMouseEvent,
				       X->srcIdx, (X->evtId = MOUSE_ENTERED),
				       0, (int)evt->x, (int)evt->y);
  }
  return ( TRUE);
}

gint leave_notify_callback( GtkWidget* widget, GdkEventCrossing* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, MouseEvent, queueMouseEvent,
				       X->srcIdx, (X->evtId = MOUSE_EXITED),
				       0, (int)evt->x, (int)evt->y);
  }
  return ( TRUE);
}

gint configure_callback( GtkWidget* widget, GdkEventConfigure* evt, gpointer data) {
  int x = evt->x;
  int y = evt->y;
  int w = evt->width;
  int h = evt->height;
 
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, ComponentEvent, queueComponentEvent,
				       X->srcIdx, (X->evtId = COMPONENT_RESIZED),
				       x, y, w, h);
  }
  return ( TRUE);
}

gint focus_in_callback( GtkWidget* widget, GdkEventFocus* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, FocusEvent, queueFocusEvent,
				       X->srcIdx, (X->evtId = FOCUS_GAINED));
  }
  return ( TRUE);
}

gint focus_out_callback( GtkWidget* widget, GdkEventFocus* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, FocusEvent, queueFocusEvent,
				       X->srcIdx, (X->evtId = FOCUS_LOST));
  }
  return ( TRUE);
}

gint map_callback( GtkWidget* widget, GdkEventAny* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, WindowEvent, queueWindowEvent,
				       X->srcIdx, (X->evtId = WINDOW_DEICONIFIED) );
  }
  return ( TRUE);
}

gint unmap_callback( GtkWidget* widget, GdkEventAny* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, WindowEvent, queueWindowEvent,
				       X->srcIdx, (X->evtId = WINDOW_ICONIFIED) );
  }
  return ( TRUE);
}

gint visibility_notify_callback( GtkWidget* widget, GdkEventVisibility* evt, gpointer data) {
  return ( TRUE);
}

gint client_callback( GtkWidget* widget, GdkEventClient* evt, gpointer data) {
  if ( getWidgetIdx( widget, (GdkEvent*)evt ) > -1 ) {
  }
  return ( TRUE);
}



/*******************************************************************************
 *
 */

jobject
Java_java_awt_Toolkit_evtInit ( JNIEnv* env, jclass clazz )
{
  jclass Component;

  ComponentEvent = (*env)->FindClass( env, "java/awt/event/ComponentEvent");
  MouseEvent     = (*env)->FindClass( env, "java/awt/event/MouseEvent");
  FocusEvent     = (*env)->FindClass( env, "java/awt/event/FocusEvent");
  WindowEvent    = (*env)->FindClass( env, "java/awt/event/WindowEvent");
  KeyEvent       = (*env)->FindClass( env, "java/awt/event/KeyEvent");
  PaintEvent     = (*env)->FindClass( env, "java/awt/event/PaintEvent");
  ActionEvent    = (*env)->FindClass( env, "java/awt/event/ActionEvent");
  ItemEvent      = (*env)->FindClass( env, "java/awt/event/ItemEvent");

  queueComponentEvent = (*env)->GetStaticMethodID( env, ComponentEvent, "queueComponentEvent", "(IIIIII)V");
  queueMouseEvent     = (*env)->GetStaticMethodID( env, MouseEvent, "queueMouseEvent", "(IIIII)V");
  queueFocusEvent     = (*env)->GetStaticMethodID( env, FocusEvent, "queueFocusEvent", "(II)V");
  queueWindowEvent    = (*env)->GetStaticMethodID( env, WindowEvent, "queueWindowEvent", "(II)V");
  queueKeyEvent       = (*env)->GetStaticMethodID( env, KeyEvent, "queueKeyEvent", "(IIIII)V");
  queuePaintEvent     = (*env)->GetStaticMethodID( env, PaintEvent, "queuePaintEvent", "(IIIIII)V");
  queueActionEvent    = (*env)->GetStaticMethodID( env, ActionEvent, "queueActionEvent", "(IILjava/lang/String;)V");
  queueItemEvent      = (*env)->GetStaticMethodID( env, ItemEvent, "queueItemEvent", "(III)V");

  X->nWindows = 300;
  X->windows = (Widget**)calloc( X->nWindows, sizeof(Widget*));

  Component = (*env)->FindClass( env, "java/awt/Component");

  return (*env)->NewObjectArray( env, X->nWindows, Component, NULL);
}


/*
 * This is just used to wakeup the getNextEvent call if we are multithreaded,
 * and we post an event from outside the dispatcher thread (which might be blocked
 * in a native getNextEvent). This is a tribute to the fact that we actually have
 * two different queues - the native one, and the queue for self-posted events
 */
void
Java_java_awt_Toolkit_evtWakeup ( JNIEnv* env, jclass clazz )
{
  GdkEventClient event;

  event.message_type = WAKEUP;
  event.data_format = 8;

  gdk_event_send_clientmessage_toall( (GdkEvent*)&event);
  //  XSendEvent( X->dsp, X->event.xany.window, False, -1, &event);
}

jint
Java_java_awt_Toolkit_evtRegisterSource ( JNIEnv* env, jclass clazz, void* wnd )
{
  register i;
  int      n;

  /*
   * we don't use a double hashing here because collisions are very unlikely
   * (window IDs usually already are hashed, so it does not make sense to
   * hash them again - we just could make it worse
   */
  for ( i = (unsigned long)wnd, n=0; n < X->nWindows; i++, n++ ) {
	i %= X->nWindows;
	if ( X->windows[i] == NULL ) {
	  X->windows[i] = (Widget*)wnd;
	  return i;
	}
  }
	
  fprintf( stderr, "window table out of space: %d", X->nWindows);
  return -1;
}


jint
Java_java_awt_Toolkit_evtUnregisterSource ( JNIEnv* env, jclass clazz, void* wnd )
{
  int i = getSourceIdx( X, (Widget*)wnd);

  if ( i >= 0 )
	X->windows[i] = NULL;

  return i;
}
