/**
 * gra.c - 
 *
 * Copyright (c) 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#include "config.h"
#include "toolkit.h"
#include "gdk/gdkprivate.h"

typedef struct {
  GdkGC*        gc;
  GdkDrawable*  drw;
  int           fg; 
  int           bg;
  GdkFont*      fnt;
  char          xor;
  int           xclr;
  int           x0;
  int           y0;
} Graphics;

#define  WINDOW   0
#define  IMAGE    1
#define  GRAPHICS 2

/*******************************************************************************
 *
 */


void*
Java_java_awt_Toolkit_graInitGraphics ( JNIEnv* env, jclass clazz,
					Graphics* gr, jobject tgt, jint tgtType,
					jint xOff, jint yOff,
					jint xClip, jint yClip, jint wClip, jint hClip,
					jobject fnt, jint fg, jint bg, jboolean blank )
{
  GdkDrawable*   drw;
  GdkGCValues    values;
  GdkRectangle   rect;
  GdkColor       c;
  GdkColormap*   map = gdk_colormap_get_system();
  unsigned long  valueMask = GDK_GC_FOREGROUND | GDK_GC_BACKGROUND | GDK_GC_FUNCTION | GDK_GC_FONT;

  if ( tgt ) {
	switch ( tgtType ) {
	case 0:  drw = ((Widget*)tgt)->wnd; break;          /* Window   */
	case 1:  drw = ((Image*)tgt)->pix; break;     /* Image    */
	case 2:  drw = ((Graphics*)tgt)->drw; break;  /* Graphics */
	default: fprintf( stderr, "illegal Graphics target: %d\n", tgtType);
		 drw = 0;
	}
  }
  else {
	if ( gr )
	  drw = gr->drw;
	else {
	  fprintf( stderr, "attempt to set an uninitialized graphics");
	  drw = 0;
	}
  }

  values.foreground.pixel = fg;
  values.background.pixel = bg;
  values.font             = (GdkFont*)fnt;
  values.function         = GDK_COPY;
  
  if ( gr ) {
    gdk_gc_set_foreground( gr->gc, &values.foreground);
    gdk_gc_set_background( gr->gc, &values.background);
    gdk_gc_set_font( gr->gc, values.font);
    gdk_gc_set_function( gr->gc, GDK_COPY);
  }
  else {
	gr = (Graphics*) malloc( sizeof(Graphics));
	gr->gc = gdk_gc_new_with_values( drw, &values, valueMask);
  }

  rect.x      = xClip; 
  rect.y      = yClip;
  rect.width  = (wClip > 0) ? wClip : 0; 
  rect.height = (hClip > 0) ? hClip : 0;
  gdk_gc_set_clip_origin( gr->gc, xOff, yOff );
  gdk_gc_set_clip_rectangle( gr->gc, &rect);
  //  XSetClipRectangles( X->dsp, gr->gc, xOff, yOff, &rect, 1, Unsorted);

  gr->fg   = fg;
  gr->bg   = bg;
  gr->drw  = drw;
  gr->x0   = xOff;
  gr->y0   = yOff;
  gr->xor  = 0;

  if ( blank ) {
	gdk_gc_set_foreground( gr->gc, &values.background);
	gdk_draw_rectangle( gr->drw, gr->gc, 1, xOff+xClip, yOff+yClip, wClip, hClip);
	gdk_gc_set_foreground( gr->gc, &values.foreground);	
  }

  return gr;
}


void
Java_java_awt_Toolkit_graFreeGraphics ( JNIEnv* env, jclass clazz, Graphics* gr )
{
  gdk_gc_destroy( gr->gc);
  free( gr);
}


void
Java_java_awt_Toolkit_graCopyArea ( JNIEnv* env, jclass clazz, Graphics* gr, 
				    jint x, jint y, jint width, jint height,
				    jint xDelta, jint yDelta )
{
  x += gr->x0; y += gr->y0;
  gdk_window_copy_area( gr->drw, gr->gc, x + xDelta, y + yDelta, gr->drw, x, y, width, height );
  //  XCopyArea( X->dsp, gr->drw, gr->drw, gr->gc, x, y, width, height, x + xDelta, y + yDelta);
}


void
Java_java_awt_Toolkit_graClearRect ( JNIEnv* env, jclass clazz, Graphics* gr,
				     jint x, jint y, jint width, jint height )
{
  GdkColor c;
  gdk_gc_set_foreground( gr->gc, getColor(gr->bg,&c));
  gdk_draw_rectangle( gr->drw, gr->gc, 1, x+gr->x0, y+gr->y0, width, height);
  gdk_gc_set_foreground( gr->gc, getColor(gr->fg,&c));	

  // XSetForeground( X->dsp, gr->gc, gr->bg);
  // XFillRectangle( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height);
  // XSetForeground( X->dsp, gr->gc, gr->fg);
}


void
Java_java_awt_Toolkit_graDrawBytes ( JNIEnv* env, jclass clazz, Graphics* gr,
				     jarray jBytes, jint offset, jint len, jint x, jint y )
{
  jboolean isCopy;
  int      n;
  jbyte    *jb;

  if ( !jBytes ) return;

  n = (*env)->GetArrayLength( env, jBytes);
  jb = (*env)->GetByteArrayElements( env, jBytes, &isCopy);
  
  if ( offset+len > n ) len = n - offset;

  gdk_draw_text( gr->drw, gr->fnt, gr->gc, x+gr->x0, y+gr->y0, (jb+offset), len);

  (*env)->ReleaseByteArrayElements( env, jBytes, jb, JNI_ABORT);
}


void
Java_java_awt_Toolkit_graDrawChars ( JNIEnv* env, jclass clazz, Graphics* gr,
				     jarray jChars, jint offset, jint len, jint x, jint y )
{
  /*
  jboolean isCopy;
  int      n;
  jchar    *jc;
  XChar2b  *b;

  if ( !jChars ) return;

  n   = (*env)->GetArrayLength( env, jChars);
  jc = (*env)->GetCharArrayElements( env, jChars, &isCopy);

  if ( offset+len > n ) len = n - offset;

#ifndef WORDS_BIGENDIAN
  n = sizeof(XChar2b)*len;
  b = (XChar2b*) getBuffer( X, n);
  swab( (jc+offset), b, n);
#else
  b = (XChar2b*) (jc + offset);
#endif

  gdk_draw_text( gr->drw, gr->fnt, gr->gc, x+gr->x0, y+gr->y0, b, len);
  //  XDrawString16( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, b, len);  

  (*env)->ReleaseCharArrayElements( env, jChars, jc, JNI_ABORT);
  */
}


void
Java_java_awt_Toolkit_graDrawString ( JNIEnv* env, jclass clazz,
				      Graphics* gr, jstring str, jint x, jint y )
{
  /*
  jboolean     isCopy;
  int          n, len;
  const jchar  *jc;
  XChar2b      *b;

  if ( !str ) return;

  len = (*env)->GetStringLength( env, str);
  jc = (*env)->GetStringChars( env, str, &isCopy);

#ifndef WORDS_BIGENDIAN
  n = sizeof(XChar2b)*len;
  b = (XChar2b*) getBuffer( X, n);
  swab( jc, b, n);
#else
  b = (XChar2b*) jc;
#endif

  gdk_draw_text( gr->drw, gr->fnt, gr->gc, x+gr->x0, y+gr->y0, b, len);
  //  XDrawString16( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, b, len);  

  (*env)->ReleaseStringChars( env, str, jc);
  */
}


void
Java_java_awt_Toolkit_graDrawLine ( JNIEnv* env, jclass clazz, Graphics* gr,
				    jint x1, jint y1, jint x2, jint y2 )
{
  gdk_draw_line( gr->drw, gr->gc, x1+gr->x0, y1+gr->y0, x2+gr->x0, y2+gr->y0);
  //  XDrawLine( X->dsp, gr->drw, gr->gc, x1 + gr->x0, y1 + gr->y0, x2 + gr->x0, y2 + gr->y0);
}


void
Java_java_awt_Toolkit_graDrawArc ( JNIEnv* env, jclass clazz, Graphics* gr,
				   jint x, jint y, jint width, jint height,
				   jint angle1, jint angle2 )
{
  gdk_draw_arc( gr->drw, gr->gc, 0, x+gr->x0, y+gr->y0, width, height, angle1<<6, angle2<<6);
  //  XDrawArc( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height, angle1<<6, angle2<<6);
}


void
Java_java_awt_Toolkit_graFillArc ( JNIEnv* env, jclass clazz, Graphics* gr,
				   jint x, jint y, jint width, jint height,
				   jint angle1, jint angle2 )
{
  gdk_draw_arc( gr->drw, gr->gc, 1, x+gr->x0, y+gr->y0, width, height, angle1<<6, angle2<<6); 
    //  XFillArc( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height, angle1<<6, angle2<<6);
}


void
Java_java_awt_Toolkit_graDrawOval ( JNIEnv* env, jclass clazz, Graphics* gr,
				    jint x, jint y, jint width, jint height )
{
  gdk_draw_arc( gr->drw, gr->gc, 0, x+gr->x0, y+gr->y0, width, height, 0, 23040); 
    //  XDrawArc( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height, 0, 23040);
}


void
Java_java_awt_Toolkit_graFillOval ( JNIEnv* env, jclass clazz, Graphics* gr,
				    jint x, jint y, jint width, jint height )
{
  gdk_draw_arc( gr->drw, gr->gc, 1, x+gr->x0, y+gr->y0, width, height, 0, 23040); 
    //  XFillArc( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height, 0, 23040);
}


int jarray2Points ( JNIEnv* env, Toolkit* X, GdkPoint** pp, int x0, int y0,
		    jarray xPoints, jarray yPoints, int nPoints )
{
  register i;
  int      n;
  jboolean isCopy;
  jint     *jx = (*env)->GetIntArrayElements( env, xPoints, &isCopy);
  jint     *jy = (*env)->GetIntArrayElements( env, yPoints, &isCopy);

  if ( (n = (*env)->GetArrayLength( env, xPoints)) < nPoints ) nPoints = n;
  if ( (n = (*env)->GetArrayLength( env, yPoints)) < nPoints ) nPoints = n;

  *pp = getBuffer( X, (nPoints+1)*sizeof(GdkPoint));

  for ( i=0; i<nPoints; i++){
	(*pp)[i].x = jx[i] + x0;
    (*pp)[i].y = jy[i] + y0;
  }

  (*env)->ReleaseIntArrayElements( env, xPoints, jx, JNI_ABORT);
  (*env)->ReleaseIntArrayElements( env, yPoints, jy, JNI_ABORT);

  return nPoints;
}


void
Java_java_awt_Toolkit_graDrawPolygon ( JNIEnv* env, jclass clazz, Graphics* gr,
				       jarray xPoints, jarray yPoints, jint nPoints )
{
  int n;
  GdkPoint   *p;

  if ( !xPoints || !yPoints ) return;

  nPoints = jarray2Points( env, X, &p, gr->x0, gr->y0, xPoints, yPoints, nPoints);

  n = nPoints-1;
  if ( (p[0].x != p[n].x) || (p[0].y != p[n].y) ) {
	p[nPoints].x = p[0].x;
	p[nPoints].y = p[0].y;
	nPoints++;
  }

  gdk_draw_lines( gr->drw, gr->gc, p, nPoints );
  //  XDrawLines( X->dsp, gr->drw, gr->gc, p, nPoints, CoordModeOrigin);
}


void
Java_java_awt_Toolkit_graDrawPolyline ( JNIEnv* env, jclass clazz, Graphics* gr,
					jarray xPoints, jarray yPoints, jint nPoints )
{
  GdkPoint   *p;

  if ( !xPoints || !yPoints ) return;

  nPoints = jarray2Points( env, X, &p, gr->x0, gr->y0, xPoints, yPoints, nPoints);
  gdk_draw_lines( gr->drw, gr->gc, p, nPoints );
  //  XDrawLines( X->dsp, gr->drw, gr->gc, p, nPoints, CoordModeOrigin);
}


void
Java_java_awt_Toolkit_graFillPolygon ( JNIEnv* env, jclass clazz, Graphics* gr,
				       jarray xPoints, jarray yPoints, jint nPoints )
{
  GdkPoint   *p;

  if ( !xPoints || !yPoints ) return;

  nPoints = jarray2Points( env, X, &p, gr->x0, gr->y0, xPoints, yPoints, nPoints);
  gdk_draw_polygon( gr->drw, gr->gc, 1, p, nPoints);
  //  XFillPolygon( X->dsp, gr->drw, gr->gc, p, nPoints, Nonconvex, CoordModeOrigin);
}


void
Java_java_awt_Toolkit_graDrawRect ( JNIEnv* env, jclass clazz, Graphics* gr,
				    jint x, jint y, jint width, jint height )
{
  gdk_draw_rectangle( gr->drw, gr->gc, 0, x+gr->x0, y+gr->y0, width, height);
  //  XDrawRectangle( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height);
}


void
Java_java_awt_Toolkit_graFillRect ( JNIEnv* env, jclass clazz, Graphics* gr,
									jint x, jint y, jint width, jint height )
{
  gdk_draw_rectangle( gr->drw, gr->gc, 1, x+gr->x0, y+gr->y0, width, height);
}



/*
 * we don't use the Xmu routines here because it would drag the whole Xt libs in
 */
void
Java_java_awt_Toolkit_graDrawRoundRect ( JNIEnv* env, jclass clazz, Graphics* gr,
					 jint x, jint y, jint width, jint height, 
					 jint wArc, jint hArc )
{
  int x1, x2, y1, y2, a, b;
  
  x += gr->x0;  y += gr->y0;
  a = wArc / 2; b = hArc / 2;

  x1 = x + a;   x2 = x + width - a;  y2 = y + height;
  gdk_draw_line( gr->drw, gr->gc, x1, y, x2, y);
  gdk_draw_line( gr->drw, gr->gc, x1, y2, x2, y2);

  y1 = y + b;   y2 = y + height - b; x2 = x + width;
  gdk_draw_line( gr->drw, gr->gc, x, y1, x, y2);
  gdk_draw_line( gr->drw, gr->gc, x2, y1, x2, y2);

  gdk_draw_arc( gr->drw, gr->gc, 0, x, y, wArc, hArc, 90*64, 90*64);

  x2 = x + width - wArc;
  gdk_draw_arc( gr->drw, gr->gc, 0, x2, y, wArc, hArc, 0, 90*64);

  y2 = y + height - hArc;
  gdk_draw_arc( gr->drw, gr->gc, 0, x2, y2, wArc, hArc, 0, -90*64);

  gdk_draw_arc( gr->drw, gr->gc, 0, x, y2, wArc, hArc, 180*64, 90*64);
}

void
Java_java_awt_Toolkit_graFillRoundRect ( JNIEnv* env, jclass clazz, Graphics* gr,
					 jint x, jint y, jint width, jint height, 
					 jint wArc, jint hArc )
{
  int x1, x2, y1, y2, a, b;
  
  x += gr->x0;  y += gr->y0;
  a = wArc / 2; b = hArc / 2;

  y1 = y + b;  y2 = y + height - b;
  if ( y2 > y1 )
	gdk_draw_rectangle( gr->drw, gr->gc, 1, x, y1, width, y2-y1);
  
  x1 = x + a;  x2 = x + width - a;
  if ( x2 > x1 ) {
	gdk_draw_rectangle( gr->drw, gr->gc, 1, x1, y, (x2-x1), b);
	gdk_draw_rectangle( gr->drw, gr->gc, 1, x1, y2, (x2-x1), b);
  }

  gdk_draw_arc( gr->drw, gr->gc, 1, x, y, wArc, hArc, 90*64, 90*64);

  x2 = x + width - wArc;
  gdk_draw_arc( gr->drw, gr->gc, 1, x2, y, wArc, hArc, 0, 90*64);

  y2 = y + height - hArc;
  gdk_draw_arc( gr->drw, gr->gc, 1, x2, y2, wArc, hArc, 0, -90*64);

  gdk_draw_arc( gr->drw, gr->gc, 1, x, y2, wArc, hArc, 180*64, 90*64);
}


void
Java_java_awt_Toolkit_graDraw3DRect ( JNIEnv* env, jclass clazz, Graphics* gr,
				      jint x, jint y, jint width, jint height,
				      jboolean raised, jint rgb )
{
  int      bright, dark;
  int      xw, yh;
  GdkColor c;

  /* we should use the same mechanism like ordinary Graphics brighter()/darker() here */
  dark   = (int) (Java_java_awt_Toolkit_clrDark( env, clazz, rgb) >> 32);
  bright = (int) (Java_java_awt_Toolkit_clrBright( env, clazz, rgb) >> 32);

  x += gr->x0; y += gr->y0;
  xw = x + width;
  yh = y + height;

  gdk_gc_set_foreground( gr->gc, (raised ? getColor( bright,&c) : getColor(dark,&c)));
  //  XSetForeground( X->dsp, gr->gc, (raised ? bright : dark));
  gdk_draw_line( gr->drw, gr->gc, x, y, xw-1, y);
  gdk_draw_line( gr->drw, gr->gc, x, y, x, yh);

  gdk_gc_set_foreground( gr->gc, (raised ? getColor(dark,&c) : getColor(bright,&c)));
  //  XSetForeground( X->dsp, gr->gc, (raised ? dark : bright));
  gdk_draw_line( gr->drw, gr->gc, x+1, yh, xw, yh);
  gdk_draw_line( gr->drw, gr->gc, xw, y, xw, yh);

  gdk_gc_set_foreground( gr->gc, getColor(gr->fg,&c));
  //  XSetForeground( X->dsp, gr->gc, gr->fg);
}

void
Java_java_awt_Toolkit_graFill3DRect ( JNIEnv* env, jclass clazz, Graphics* gr,
				      jint x, jint y, jint width, jint height,
				      jboolean raised, jint rgb )
{
  gdk_draw_rectangle( gr->drw, gr->gc, 1, x+gr->x0+1, y+gr->y0+1, width-2, height-2);

  Java_java_awt_Toolkit_graDraw3DRect( env, clazz, gr, x, y, width-1, height-1, raised, rgb);
}


void
Java_java_awt_Toolkit_graAddClip ( JNIEnv* env, jclass clazz, Graphics* gr,
			           jint xClip, jint yClip, jint wClip, jint hClip )
{
	/* not implemented yet */
}


void
Java_java_awt_Toolkit_graSetClip ( JNIEnv* env, jclass clazz, Graphics* gr,
				   jint xClip, jint yClip, jint wClip, jint hClip )
{
  GdkRectangle rect;

  rect.x      = xClip;
  rect.y      = yClip;
  rect.width  = (wClip > 0) ? wClip : 0; 
  rect.height = (hClip > 0) ? hClip : 0;
  gdk_gc_set_clip_origin( gr->gc, gr->x0, gr->y0);
  gdk_gc_set_clip_rectangle( gr->gc, &rect);
  //  XSetClipRectangles( X->dsp, gr->gc, gr->x0, gr->y0, &rect, 1, Unsorted);
}

void
Java_java_awt_Toolkit_graSetColor ( JNIEnv* env, jclass clazz, Graphics* gr, jint clr )
{
  GdkColor c;
  gr->fg = clr;

  if ( gr->xor )
	clr ^= gr->xclr;

  gdk_gc_set_foreground( gr->gc, getColor( clr,&c));
  //  XSetForeground( X->dsp, gr->gc, clr);
}

void
Java_java_awt_Toolkit_graSetBackColor ( JNIEnv* env, jclass clazz, Graphics* gr, jint clr )
{
  GdkColor c;
  gr->bg = clr;
  gdk_gc_set_background( gr->gc, getColor(clr,&c));
  //  XSetBackground( X->dsp, gr->gc, clr);
}

void
Java_java_awt_Toolkit_graSetFont ( JNIEnv* env, jclass clazz, Graphics* gr, jobject fnt )
{
  gdk_gc_set_font( gr->gc, (GdkFont*)fnt);
  //  XSetFont( X->dsp, gr->gc, ((XFontStruct*)fnt)->fid);
}

void
Java_java_awt_Toolkit_graSetOffset ( JNIEnv* env, jclass clazz, Graphics* gr, jint xOff, jint yOff )
{
  gr->x0 = xOff;
  gr->y0 = yOff;
}

void
Java_java_awt_Toolkit_graSetPaintMode ( JNIEnv* env, jclass clazz, Graphics* gr )
{
  GdkColor c;
  gr->xor = 0;
  gdk_gc_set_foreground( gr->gc, getColor(gr->fg,&c));
  gdk_gc_set_function( gr->gc, GDK_COPY);
}

void
Java_java_awt_Toolkit_graSetXORMode ( JNIEnv* env, jclass clazz, Graphics* gr, jint xorClr )
{
  GdkColor c;
  gr->xor = 1;
  gr->xclr = xorClr;
  gdk_gc_set_foreground( gr->gc, getColor( gr->fg ^ xorClr, &c));
  gdk_gc_set_function( gr->gc, GDK_XOR);
}


/************************************************************************************
 * image rendering
 */

void
drawAlphaImage ( Graphics* gr, Image* img,
		 jint srcX, jint srcY, jint dstX, jint dstY,
		 jint width, jint height, jint bgval )
{
  /*
  GdkImage *dstImg;
  int    i, j, si, sj, alpha;
  unsigned long dpix, spix, bgpix;
  int    sr, sg, sb, dr, dg, db;

  dstImg = gdk_image_get( gr->drw, dstX, dstY, width, height );
  if ( bgval != -1 )
	bgpix = pixelValue( X, bgval);

  if ( dstImg ) {
	for ( j=0, sj=srcY; j<dstImg->height; j++, sj++ ) {
	  for ( i=0, si=srcX; i<dstImg->width; i++, si++ ) {
		dpix = (bgval == -1) ? gdk_image_get_pixel( dstImg, i, j) : bgpix;
		spix = gdk_image_get_pixel( img->xImg, si, sj);
		alpha = GetAlpha( img->alpha, si, sj);

		rgbValues( X, dpix, &dr, &dg, &db);
		rgbValues( X, spix, &sr, &sg, &sb);

		dr = ((alpha * sr + (255 - alpha) * dr) + 128) >> 8;
		dg = ((alpha * sg + (255 - alpha) * dg) + 128) >> 8;
		db = ((alpha * sb + (255 - alpha) * db) + 128) >> 8;

		gdk_image_put_pixel( dstImg, i, j, pixelValue( X, (dr << 16)|(dg << 8)|(db) ));
	  }
	}

	gdk_draw_image( gr->drw, gr->gc, dstImg, 0, 0, dstX, dstY, width, height);
	gdk_image_destroy( dstImg);
	//	XPutImage( X->dsp, gr->drw, gr->gc, dstImg, 0, 0, dstX, dstY, width, height);
	//	XDestroyImage( dstImg);
  }
  */
}


void
Java_java_awt_Toolkit_graDrawImage ( JNIEnv* env, jclass clazz, Graphics* gr, Image* img,
				     jint srcX, jint srcY,
				     jint dstX, jint dstY,
				     jint width, jint height, jint bgval )
{
  XGCValues values;
  GdkColor cb;
  GdkGCPrivate *private;

  private = (GdkGCPrivate*) gr->gc;
  dstX += gr->x0;
  dstY += gr->y0;

  if ( img->pix ) {
        gdk_draw_pixmap( gr->drw, gr->gc, img->pix, srcX, srcY, dstX, dstY, width, height);
  }
  else if ( img->xImg ) {
        if ( img->xMask ) {
          /* save current gc values, we have to change them temporarily */
          XGetGCValues( private->xdisplay, private->xgc, GCFunction|GCPlaneMask|GCForeground|GCBackground, &values);
	  //	  gdk_gc_get_values( gr->gc, &values);

          /* in case we have a specified background color, fill area */
          if ( bgval != -1 ) {
	    cb.pixel = bgval;
            gdk_gc_set_foreground( gr->gc, &cb);
            gdk_draw_rectangle( gr->drw, gr->gc, TRUE, dstX, dstY, width, height);
          }
         /* set attributes so that a subsequent image draw leaves the mask pels unmodified */
          XSetFunction( private->xdisplay, private->xgc, GXandInverted);
          XSetForeground( private->xdisplay, private->xgc, 0xffffffff);
          XSetBackground( private->xdisplay, private->xgc, 0);

          /* draw the mask bitmap */
	  gdk_draw_image( gr->drw, gr->gc, img->xMask, srcX, srcY, dstX, dstY, width, height);
	  //XPutImage( private->xdisplay, gr->drw, gr->gc, img->xMask, srcX, srcY, dstX, dstY, width, height);

          /* restore gc values except of the function */
          XChangeGC( private->xdisplay, private->xgc, GCFunction|GCPlaneMask|GCForeground|GCBackground, &values);
          XSetFunction( private->xdisplay, private->xgc, GXor);
        }
        else if ( img->alpha ) {
          drawAlphaImage( gr, img, srcX, srcY, dstX, dstY, width, height, bgval);
          return;
        }

        /* this is the real image drawing */
	gdk_draw_image( gr->drw, gr->gc, img->xImg, srcX, srcY, dstX, dstY, width, height);
	//       XPutImage( X->dsp, gr->drw, gr->gc, img->xImg, srcX, srcY, dstX, dstY, width, height);

        /* in case we had a mask, restore the original function */
        if ( img->xMask )
             XSetFunction( private->xdisplay, private->xgc, values.function);
  }

  //  XFLUSH( X, True);
}


void
Java_java_awt_Toolkit_graDrawImageScaled ( JNIEnv* env, jclass clazz, Graphics* gr, Image* img,
					   jint dx0, jint dy0, jint dx1, jint dy1,
					   jint sx0, jint sy0, jint sx1, jint sy1, jint bgval )
{

  Java_java_awt_Toolkit_graDrawImage( env, clazz, gr, img, sx0, sy0, dx0, dy0, sx1-sx0, sy1-sy0, bgval);
  /*
  int        x0, y0, x1, y1;
  int        iw = img->xImg->width;
  Image      tgt;

  dx0 += gr->x0;
  dx1 += gr->x0;
  dy0 += gr->y0;
  dy1 += gr->y0;

  if ( dx1 > dx0 ) {
        x0 = dx0; x1 = dx1;
  }
  else {
        x0 = dx1; x1 = dx0;
  }
  if ( dy1 > dy0 ) {
        y0 = dy0; y1 = dy1;
 }
  else {
        y0 = dy1; y1 = dy0;
  }

  if ( sx0 < 0 ) sx0 = 0;
  if ( sx1 < 0 ) sx1 = 0;
  if ( sx0 >= iw ) sx0 = iw-1;
  if ( sx1 >= iw ) sx1 = iw-1;

  tgt.pix = 0;
  tgt.width = (x1 - x0 +1);
  tgt.height = (y1 - y0 +1);
  tgt.xImg = createXImage( X, tgt.width, tgt.height);
  tgt.xMask = (img->xMask) ? createXMaskImage( X, tgt.width, tgt.height) : 0;
  tgt.alpha = (img->alpha) ? createAlphaImage( X, tgt.width, tgt.height) : 0;

  initScaledImage( X, &tgt, img, dx0-x0, dy0-y0, dx1-x0, dy1-y0, sx0, sy0, sx1, sy1);

  Java_java_awt_Toolkit_graDrawImage ( env, clazz, gr, &tgt, 0, 0,
                                      x0, y0, tgt.width, tgt.height, bgval);

  // since malloc / free might be redirected, we better clean up manually
  gdk_image_destroy( tgt.xImg);

  if ( tgt.xMask != NULL ) {
    gdk_image_destroy( tgt.xMask);
  }
  */
}
