/*
 * Copyright (c) 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */


#include <stdio.h>
#include <gtk/gtk.h>
#include <toolkit.h>


void
select_row_callback ( GtkWidget* widget, gint row, gint col, GdkEvent* evt, gpointer data ) {
  if ( getWidgetIdx( widget, evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, ItemEvent, queueItemEvent,
				       X->srcIdx, (X->evtId = ITEM_SELECTED), row );
  }
}

void
unselect_row_callback ( GtkWidget* widget, gint row, gint col, GdkEvent* evt, gpointer data ) {
  if ( getWidgetIdx( widget, evt ) > -1 ) {
    (*X->env)->CallStaticObjectMethod( X->env, ItemEvent, queueItemEvent,
				       X->srcIdx, (X->evtId = ITEM_DESELECTED), row );
  }
}


void
connect_lst_signals( GtkWidget* w) {
  gtk_signal_connect( GTK_OBJECT( w), "select_row", GTK_SIGNAL_FUNC( select_row_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( w), "unselect_row", GTK_SIGNAL_FUNC( unselect_row_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( w), "key_press_event", GTK_SIGNAL_FUNC( key_press_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( w), "button_release_event", GTK_SIGNAL_FUNC( button_release_callback), NULL);
}

void*
Java_java_awt_Toolkit_lstCreateList ( JNIEnv* env, jclass clazz )
{
  Widget *widget = calloc( 1, sizeof( Widget));
  GtkCList* lst = gtk_clist_new( 1);

  widget->widget = lst;

  connect_lst_signals( lst);
  gtk_clist_set_policy( lst, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  return (void*)widget;
}

void
Java_java_awt_Toolkit_lstAppendItem( JNIEnv* env, jclass clazz, void* nd, jstring itm)
{
  GtkWidget* lst = ((Widget*)nd)->widget;
  char* buf[1];

  buf[0] = java2CString( env, X, itm);

  gtk_clist_append( GTK_CLIST( lst), buf);
}

void
Java_java_awt_Toolkit_lstInsertItem( JNIEnv* env, jclass clazz, void* nd, jstring itm, jint row)
{
  GtkCList* lst = GTK_CLIST( ((Widget*)nd)->widget);
  char* buf[1];

  buf[0] = java2CString( env, X, itm);

  gtk_clist_insert( GTK_CLIST( lst), row, buf);
}

void
Java_java_awt_Toolkit_lstUnselectItem( JNIEnv* env, jclass clazz, void* nd, jint row)
{
  GtkCList* lst = GTK_CLIST( ((Widget*)nd)->widget);
  gtk_clist_unselect_row( lst, row, 0);
}

void
Java_java_awt_Toolkit_lstRemoveAll( JNIEnv* env, jclass clazz, void* nd )
{
  GtkCList* lst = GTK_CLIST( ((Widget*)nd)->widget);
  gtk_clist_clear( lst);
}

void
Java_java_awt_Toolkit_lstRemoveItem( JNIEnv* env, jclass clazz, void* nd, jint row)
{
  GtkCList* lst = GTK_CLIST( ((Widget*)nd)->widget);
  gtk_clist_remove( lst, row);
}

void
Java_java_awt_Toolkit_lstReplaceItem( JNIEnv* env, jclass clazz, void* nd, jstring itm, jint row)
{
  GtkCList* lst = GTK_CLIST( ((Widget*)nd)->widget);
  char* buf = java2CString( env, X, itm);

  gtk_clist_set_text( lst, row, 0, buf);
}

void
Java_java_awt_Toolkit_lstSelectItem( JNIEnv* env, jclass clazz, void* nd, jint row)
{
  GtkCList* lst = GTK_CLIST( ((Widget*)nd)->widget);
  gtk_clist_select_row( lst, row, 0);
}

void
Java_java_awt_Toolkit_lstSetMultipleMode( JNIEnv* env, jclass clazz, void* nd, jboolean mode )
{
  GtkCList* lst = GTK_CLIST( ((Widget*)nd)->widget);
  gtk_clist_set_selection_mode( lst, mode ? GTK_SELECTION_MULTIPLE : GTK_SELECTION_BROWSE );
}

void
Java_java_awt_Toolkit_lstFreeze( JNIEnv* env, jclass clazz, void* nd)
{
  GtkCList* lst = GTK_CLIST( ((Widget*)nd)->widget);
  gtk_clist_freeze( lst);
}

void
Java_java_awt_Toolkit_lstThaw( JNIEnv* env, jclass clazz, void* nd)
{
  GtkCList* lst = GTK_CLIST( ((Widget*)nd)->widget);
  gtk_clist_thaw( lst);
}
