/**
 * toolkit.h - 
 *
 * Copyright (c) 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */

#ifndef __toolkit_h
#define __toolkit_h

#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdktypes.h>
#include <jni.h>
#if defined(HAVE_STRING_H)
#include <string.h>
#endif

jclass     ComponentEvent;
jclass     MouseEvent;
jclass     FocusEvent;
jclass     WindowEvent;
jclass     KeyEvent;
jclass     PaintEvent;
jclass     ActionEvent;
jclass     ItemEvent;

jmethodID  queueComponentEvent;
jmethodID  queueMouseEvent;
jmethodID  queueFocusEvent;
jmethodID  queueWindowEvent;
jmethodID  queueKeyEvent;
jmethodID  queuePaintEvent;
jmethodID  queueItemEvent;
jmethodID  queueActionEvent;

#define    COMPONENT_RESIZED    101

#define    WINDOW_CLOSING       201
#define    WINDOW_CLOSED        202
#define    WINDOW_ICONIFIED     203
#define    WINDOW_DEICONIFIED   204

#define    KEY_PRESSED          401
#define    KEY_RELEASED         402

#define    MOUSE_PRESSED        501
#define    MOUSE_RELEASED       502
#define    MOUSE_MOVED          503
#define    MOUSE_ENTERED        504
#define    MOUSE_EXITED         505

#define    PAINT                800

#define    FOCUS_GAINED        1004
#define    FOCUS_LOST          1005

#define    ITEM_SELECTED       1
#define    ITEM_DESELECTED     2

#define    ACTION_PERFORMED    1001


#define JRGB(_r,_g,_b)  (_r<<16 | _g<<8 | _b)
#define JRED(_rgb)      ((_rgb & 0xff0000) >> 16)
#define JGREEN(_rgb)    ((_rgb & 0x00ff00) >> 8)
#define JBLUE(_rgb)     (_rgb & 0x0000ff)

#define D8   36.43  /* 255 / 7 */
#define D16  18.21

#define JI8(_v) (int)((_v + D16) / D8)
#define XI8(_v) (int)(((_v>>8) + D16) / D8)

/*******************************************************************************
 *
 */

gint button_press_callback       ( GtkWidget* widget, GdkEventButton* event, gpointer data);
gint button_release_callback     ( GtkWidget* widget, GdkEventButton* event, gpointer data);
gint motion_notify_callback      ( GtkWidget* widget, GdkEventMotion* event, gpointer data);
gint delete_callback             ( GtkWidget* widget, GdkEventAny* event, gpointer data);
gint destroy_callback            ( GtkWidget* widget, GdkEventAny* event, gpointer data);
gint expose_callback             ( GtkWidget* widget, GdkEventExpose* event, gpointer data);
gint key_press_callback          ( GtkWidget* widget, GdkEventKey* event, gpointer data);
gint key_release_callback        ( GtkWidget* widget, GdkEventKey* event, gpointer data);
gint enter_notify_callback       ( GtkWidget* widget, GdkEventCrossing* event, gpointer data);
gint leave_notify_callback       ( GtkWidget* widget, GdkEventCrossing* event, gpointer data);
gint configure_callback          ( GtkWidget* widget, GdkEventConfigure* event, gpointer data);
gint focus_in_callback           ( GtkWidget* widget, GdkEventFocus* event, gpointer data);
gint focus_out_callback          ( GtkWidget* widget, GdkEventFocus* event, gpointer data);
gint map_callback                ( GtkWidget* widget, GdkEventAny* event, gpointer data);
gint unmap_callback              ( GtkWidget* widget, GdkEventAny* event, gpointer data);
gint visibility_notify_callback  ( GtkWidget* widget, GdkEventVisibility* event, gpointer data);
gint client_callback             ( GtkWidget* widget, GdkEventClient* client, gpointer data);

gint idle_callback               ( gpointer data);
void connectSignals              ();

typedef struct _Rgb2True {
  unsigned int   redMask;
  unsigned int   greenMask;
  unsigned int   blueMask;
  char           blueShift;
  char           redShift;
  char           greenShift;
} Rgb2True;


typedef struct _RgbColor {
  unsigned char r;
  unsigned char g;
  unsigned char b;
} RgbColor;


typedef struct _Rgb2Pseudo {
  RgbColor       rgb[256];
  unsigned char  pix[8][8][8];
} Rgb2Pseudo;

typedef struct _AlphaImage {
  unsigned char *buf;
  int           width, height;
} AlphaImage;

typedef struct _Image {
  GdkPixmap      *pix;
  GdkImage       *xImg;
  GdkImage       *xMask;
  AlphaImage     *alpha;         /* alpha channel (for alpha != 0x00 or 0xff) */
  int            trans;          /* transparent index */
  int            width, height;  /* we need this in case we are a pixmap */
} Image;


typedef struct _Widget {
  GtkWidget     *widget;
  GdkWindow     *wnd;
  GtkWidget     *box;
  GdkCursor     *cursor;
  GdkFont       *fnt;
  int           bg;
  int           fg;
} Widget;

typedef struct _Toolkit {
  GtkWindow*      root;

  char           *buf;
  unsigned int   nBuf;

  JNIEnv         *env;
  GdkCursor      cursors[14];

  GdkEvent*      event;
  int            evtId;

  Widget         *lastWindow;
  int            srcIdx;
  Widget         **windows;
  int            nWindows;

  Widget         *cbdOwner;

  Widget         *newWindow;
} Toolkit;


#ifdef MAIN

Toolkit   XTk;
Toolkit   *X = &XTk;

GdkAtom WM_PROTOCOLS;
GdkAtom WM_DELETE_WINDOW;
GdkAtom WM_TAKE_FOCUS;
GdkAtom WAKEUP;
GdkAtom RETRY_FOCUS;
GdkAtom SELECTION_DATA;
GdkAtom JAVA_OBJECT;

#else

extern Toolkit* X;

extern GdkAtom WM_PROTOCOLS;
extern GdkAtom WM_DELETE_WINDOW;
extern GdkAtom WM_TAKE_FOCUS;
extern GdkAtom WAKEUP;
extern GdkAtom RETRY_FOCUS;
extern GdkAtom SELECTION_DATA;
extern GdkAtom JAVA_OBJECT;

#endif

static __inline__ GdkColor* getColor( int rgb, GdkColor *c) {
  c->red = rgb >> 16;
  c->green = rgb >> 8;
  c->blue = rgb;
  //  c->pixel = gdk_rgb_xpixel_from_rgb( rgb);

  return c;
}

static __inline__ char* java2CString ( JNIEnv *env, Toolkit* X, jstring jstr ) {
  jboolean isCopy;
  register i;
  int      n = (*env)->GetStringLength( env, jstr);
  const jchar    *jc = (*env)->GetStringChars( env, jstr, &isCopy);

  if ( n >= X->nBuf ) {
	if ( X->buf )
	  free( X->buf);
	X->buf = (char*)malloc( n+1);
	X->nBuf = n+1;
  }

  for ( i=0; i<n; i++ ) X->buf[i] = (char) jc[i];
  X->buf[i] = 0;
  (*env)->ReleaseStringChars( env, jstr, jc);

  return X->buf;
}

static __inline__ char* jchar2CString ( Toolkit* X, jchar* jc, int len ) {
  register i;
  int      n = len+1;
  
  if ( n > X->nBuf ) {
	if ( X->buf )
	  free( X->buf);
	X->buf  = (char*)malloc( n);
	X->nBuf = n;
  }

  for ( i=0; i<len; i++ ) X->buf[i] = (char) jc[i];
  X->buf[i] = 0;

  return X->buf;
}

static __inline__ void* getBuffer ( Toolkit* X, unsigned int nBytes ) {
  if ( nBytes > X->nBuf ) {
	if ( X->buf )
	  free( X->buf);
	X->buf  = (char*)malloc( nBytes);
	X->nBuf = nBytes;
  }
  return X->buf;
}


/*****************************************************************************************
 * color functions
 */

static __inline__ gulong pixelValue ( Toolkit* X, guint32 rgb)
{
  return gdk_rgb_xpixel_from_rgb( rgb);
}

static __inline__ void
rgbValues ( Toolkit* X, guint32 pixel, int* r, int* g, int* b )
{
  GdkColor *c;
  GdkColormap* cmap = gdk_rgb_get_cmap();
  int idx;

  for ( idx=0; idx<cmap->size; idx++) {
    c = &(cmap->colors[idx]);
    if ( c->pixel == pixel ) {
      *r = c->red;
      *g = c->green;
      *b = c->blue;
      return;
    }
  }
}

void initColorMapping ( JNIEnv* env, Toolkit* X);
jlong Java_java_awt_Toolkit_clrBright ( JNIEnv* env, jclass clazz, jint rgb );
jlong Java_java_awt_Toolkit_clrDark ( JNIEnv* env, jclass clazz, jint rgb );



/*****************************************************************************************
 * image functions
 */

GdkImage* createXMaskImage ( Toolkit* X, int width, int height );
GdkImage* createXImage ( Toolkit* X, int width, int height );
AlphaImage* createAlphaImage ( Toolkit* X, int width, int height );
void initScaledImage ( Toolkit* X, Image *tgt, Image *src,
		       int dx0, int dy0, int dx1, int dy1,
		       int sx0, int sy0, int sx1, int sy1 );


static __inline__ void
PutAlpha ( AlphaImage* img, int col, int row, unsigned char alpha )
{
  img->buf[ row*img->width + col ] = alpha;
}

static __inline__ int
GetAlpha ( AlphaImage* img, int col, int row )
{
  return img->buf[ row*img->width + col];
}

/*****************************************************************************************
 * clipboard functions
 */

jobject selectionClear ( JNIEnv* env, Toolkit* X );
jobject selectionRequest ( JNIEnv* env, Toolkit* X );


#endif
