/*
 * Copyright (c) 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */


#include <stdio.h>
#include <gtk/gtk.h>
#include <gtk/gtktable.h>
#include <toolkit.h>

GdkCursor*
getCursor ( jint jCursor )
{
  GdkCursorType shape;

  switch ( jCursor ) {
  case 0:  shape = GDK_TOP_LEFT_ARROW; break;         /* DEFAULT_CURSOR */
  case 1:  shape = GDK_CROSSHAIR; break;              /* CROSSHAIR_CURSOR */
  case 2:  shape = GDK_XTERM; break;                  /* TEXT_CURSOR */
  case 3:  shape = GDK_WATCH; break;                  /* WAIT_CURSOR */
  case 4:  shape = GDK_BOTTOM_LEFT_CORNER; break;     /* SW_RESIZE_CURSOR */
  case 5:  shape = GDK_BOTTOM_RIGHT_CORNER; break;    /* SE_RESIZE_CURSOR */
  case 6:  shape = GDK_TOP_LEFT_CORNER; break;        /* NW_RESIZE_CURSOR */
  case 7:  shape = GDK_TOP_RIGHT_CORNER; break;       /* NE_RESIZE_CURSOR */
  case 8:  shape = GDK_TOP_SIDE; break;               /* N_RESIZE_CURSOR */
  case 9:  shape = GDK_BOTTOM_SIDE; break;            /* S_RESIZE_CURSOR */
  case 10: shape = GDK_LEFT_SIDE; break;              /* W_RESIZE_CURSOR */
  case 11: shape = GDK_RIGHT_SIDE; break;             /* E_RESIZE_CURSOR */
  case 12: shape = GDK_HAND2; break;                  /* HAND_CURSOR */
  case 13: shape = GDK_FLEUR; break;                  /* MOVE_CURSOR */
  default: shape = GDK_TOP_LEFT_ARROW; break;
  }

  return gdk_cursor_new( shape);
}

void
Java_java_awt_Toolkit_wndSetTitle ( JNIEnv* env, jclass clazz, void* wnd, jstring jTitle )
{
  char *buf;

  if ( jTitle ) {
    buf = java2CString( env, X, jTitle);
    gtk_window_set_title( ((Widget*)wnd)->widget, buf);
  }
}

void
Java_java_awt_Toolkit_wndSetResizable ( JNIEnv* env, jclass clazz, void* wnd, jboolean isResizable, int x, int y, int width, int height )
{
  gtk_window_set_policy( ((Widget*)wnd)->widget, isResizable, isResizable, isResizable);
  /*
  gint min_w = isResizable ? 0 : width;
  gint min_h = isResizable ? 0 : height;
  gint max_w = isResizable ? INT_MAX : width;
  gint max_h = isResizable ? INT_MAX : height;
  gint flags = GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE;

  gdk_window_set_hints ( ((GtkWidget*)wnd)->window, x, y, min_w, min_h, max_w, max_h, flags );
  */
}

void
connect_wnd_signals( GtkWidget* wnd) {

  gtk_signal_connect( GTK_OBJECT( wnd), "button_press_event", GTK_SIGNAL_FUNC( button_press_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "button_release_event", GTK_SIGNAL_FUNC( button_release_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "motion_notify_event", GTK_SIGNAL_FUNC( motion_notify_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "delete_event", GTK_SIGNAL_FUNC( delete_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "expose_event", GTK_SIGNAL_FUNC( expose_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "key_press_event", GTK_SIGNAL_FUNC( key_press_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "key_release_event", GTK_SIGNAL_FUNC( key_release_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "enter_notify_event", GTK_SIGNAL_FUNC( enter_notify_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "leave_notify_event", GTK_SIGNAL_FUNC( leave_notify_callback), NULL);
    gtk_signal_connect( GTK_OBJECT( wnd), "configure_event", GTK_SIGNAL_FUNC( configure_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "focus_in_event", GTK_SIGNAL_FUNC( focus_in_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "focus_out_event", GTK_SIGNAL_FUNC( focus_out_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "map_event", GTK_SIGNAL_FUNC( map_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "unmap_event", GTK_SIGNAL_FUNC( unmap_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "visibility_notify_event", GTK_SIGNAL_FUNC( visibility_notify_callback), NULL);
  gtk_signal_connect( GTK_OBJECT( wnd), "client_event", GTK_SIGNAL_FUNC( client_callback), NULL);

  gtk_signal_connect( GTK_OBJECT( wnd), "destroy", GTK_SIGNAL_FUNC( destroy_callback), NULL);
}

Widget*
createWindow ( JNIEnv* env, jclass clazz, GtkWindowType type, GdkWindow* parent, void* owner, jstring jTitle, jint x, jint y, jint width, jint height, jint jCursor, jint clrBack, jboolean isResizable )
{
  Widget *widget = calloc( 1, sizeof( Widget));

  widget->widget = gtk_window_new( type);
  gtk_widget_set_events( widget->widget, GDK_ALL_EVENTS_MASK);

  connect_wnd_signals( widget->widget);

  if ( parent )
    gtk_widget_set_parent_window( widget->widget, parent);

  gtk_widget_set_uposition( widget->widget, x, y);
  gtk_widget_set_usize( widget->widget, width, height);

  if ( !isResizable )
    Java_java_awt_Toolkit_wndSetResizable( env, clazz, (void*) widget, isResizable, x, y, width, height );
  if ( jTitle )
    Java_java_awt_Toolkit_wndSetTitle( env, clazz, (void*) widget, jTitle );

  widget->cursor = getCursor( jCursor);
  widget->bg = clrBack;

  return widget;
}

void*
Java_java_awt_Toolkit_wndCreateFrame ( JNIEnv* env, jclass clazz, jstring jTitle, jint x, jint y,
				       jint width, jint height, jint jCursor, jint clrBack, jboolean isResizable )
{
  GdkWindow* parent = NULL;
  void* owner = NULL;

  return (void*)createWindow( env, clazz, GTK_WINDOW_TOPLEVEL, parent, owner, jTitle,
			      x, y, width, height, jCursor, clrBack, isResizable );

}

void*
Java_java_awt_Toolkit_wndCreateWindow ( JNIEnv* env, jclass clazz, void* owner, jint x, jint y,
					jint width, jint height, jint jCursor, jint clrBack )
{
  GdkWindow* parent = NULL;

  return (void*)createWindow( env, clazz, GTK_WINDOW_POPUP, parent, owner, NULL, x, y,
			      width, height, jCursor, clrBack, JNI_TRUE);
}

void*
Java_java_awt_Toolkit_wndCreateDialog ( JNIEnv* env, jclass clazz, void* owner, jstring jTitle, jint x,
					jint y, jint width, jint height, jint jCursor, jint clrBack, jboolean isResizable )
{
  GdkWindow* parent = NULL;

  return (void*)createWindow( env, clazz, GTK_WINDOW_DIALOG, parent, owner, jTitle, x, y,
			      width, height, jCursor, clrBack, isResizable );
}

void
Java_java_awt_Toolkit_wndDestroyWindow ( JNIEnv* env, jclass clazz, void* wnd )
{
  gtk_widget_destroy( ((Widget*)wnd)->widget);
}

void
Java_java_awt_Toolkit_wndRequestFocus( JNIEnv* env, jclass clazz, void* wnd )
{
}

void
Java_java_awt_Toolkit_wndSetWindowBounds( JNIEnv* env, jclass clazz, void* wnd, jint x, jint y, jint width, jint height )
{
  GdkWindow* w = ((Widget*)wnd)->widget->window;
  gdk_window_move_resize( w, x, y, width, height );
}

void
Java_java_awt_Toolkit_wndSetFrameBounds( JNIEnv* env, jclass clazz, void* wnd, jint x, jint y, jint width, jint height )
{
  GdkWindow* w = ((Widget*)wnd)->widget->window;
  gdk_window_move_resize( w, x, y, width, height );
}

void
Java_java_awt_Toolkit_wndSetFrameOffsets ( JNIEnv* env, jclass clazz, jint titleBarHeight, jint menuBarHeight, jint bottomBarHeight, jint borderWidth )
{
}

void
Java_java_awt_Toolkit_wndSetIcon( JNIEnv* env, jclass clazz, void* wnd, void* img )
{
}

void
Java_java_awt_Toolkit_wndSetVisible ( JNIEnv* env, jclass clazz, void* wnd, jboolean showIt )
{
  Widget *w = (Widget*) wnd;
  GdkColor cb;

  if ( showIt ) {
    gtk_widget_show( w->widget);
    w->wnd = w->widget->window;
    cb.pixel = w->bg;
    gdk_window_set_background( w->wnd, &cb);
    gdk_window_set_cursor( w->wnd, w->cursor);
  }
  else {
    gtk_widget_hide( w->widget);
  }
  
}

void
Java_java_awt_Toolkit_wndToFront ( JNIEnv* env, jclass clazz, void* wnd )
{
  GdkWindow* w = ((Widget*)wnd)->widget->window;
  if ( w != NULL )
    gdk_window_raise( w);
}

void
Java_java_awt_Toolkit_wndToBack ( JNIEnv* env, jclass clazz, void* wnd )
{
  GdkWindow* w = ((Widget*)wnd)->widget->window;
  if ( w != NULL )
    gdk_window_lower( w);
}

void
Java_java_awt_Toolkit_wndSetCursor ( JNIEnv* env, jclass clazz, void* wnd, jint jCursor )
{
  Widget* w = (Widget*) wnd;
  GdkWindow* wnd = w->widget->window;
  w->cursor = getCursor( jCursor);

  if ( wnd != NULL )
    gdk_window_set_cursor( wnd, w->cursor);
}


void
Java_java_awt_Toolkit_wndAddWidget ( JNIEnv* env, jclass clazz, void* wnd, void* wdg )
{
  Widget* wp = (Widget*)wnd;
  GtkWidget* gw = ((Widget*)wdg)->widget;

  if ( wp->box == NULL ) {
    wp->box = gtk_table_new( 0, 0, FALSE);
    gtk_widget_show( wp->box);
    gtk_container_add( GTK_CONTAINER( wp->widget ), wp->box );
  }

  gtk_table_attach( GTK_TABLE( wp->box), gw, 0, 1, 0, 1, 0, 0, 0, 0);
  gtk_widget_show( gw);

}
