package java.awt;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.util.Vector;

/**
 * class List - 
 *
 * Copyright (c) 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */
public class List
  extends NativeComponent
  implements ItemSelectable
{
	ActionListener aListener;
	ItemListener iListener;
	boolean multipleMode;
        Vector items = new Vector();
        Vector selections = new Vector();

public List () {
	this( 3, false);
}

public List ( int rows) {
	this( rows, false);
}

public List ( int rows, boolean multipleMode) {
	setMultipleMode( multipleMode);
	setForeground( Defaults.ListTxtClr);
	setBackground( Defaults.ListBgClr);
	setFont( Defaults.ListFont);
}

public void addNotify() {
  if ( nativeData == null ) {
    nativeData = Toolkit.lstCreateList();
    setMultipleMode( multipleMode);
    setItems();
    super.addNotify();
  }
}

public void add ( String item) {
	addElement( item, -1);
}

public synchronized void add ( String item, int index) {
	addElement( item, index);
}

public synchronized void addActionListener ( ActionListener l) {
	aListener = AWTEventMulticaster.add( aListener, l);
	eventMask |= AWTEvent.ACTION_EVENT_MASK;
}

void addElement ( String item, int index) {
  if ( index == -1 ) {
    items.addElement( item);
    if ( nativeData != null )
      Toolkit.lstAppendItem( nativeData, item);
  }
  else {
    items.insertElementAt( item, index);
    if ( nativeData != null )
      Toolkit.lstInsertItem( nativeData, item, index);
  }

}

void setItems() {
  if ( nativeData != null ) {
    Toolkit.lstFreeze( nativeData);
    for ( int i=0; i<items.size(); i++)
      Toolkit.lstAppendItem( nativeData, (String)items.elementAt( i) );
    Toolkit.lstThaw( nativeData);
  }
}

public void addItem ( String item) {
	addElement( item, -1);
}

public synchronized void addItem ( String item, int index) {
	addElement( item, index);
}

public synchronized void addItemListener ( ItemListener l) {
	iListener = AWTEventMulticaster.add( iListener, l);
	eventMask |= AWTEvent.ITEM_EVENT_MASK;
}

public synchronized void delItem ( int index) {
	removeElement( index);
}

public synchronized void deselect ( int index) {
	deselectElement( index);
}

void deselectElement ( int index ) {
      if ( nativeData != null )
	Toolkit.lstUnselectItem( nativeData, index);

}

public String getItem ( int index) {
	return (String)items.elementAt( index);
}

public int getItemCount () {
	return items.size();
}

public synchronized String[] getItems () {
	String[] si = new String[ items.size() ];
	for ( int i=0; i<si.length; i++)
		si[i] = (String)items.elementAt( i);
	return si;
}

public Dimension getMinimumSize () {
	return new Dimension( 50, 50);
}

public Dimension getMinimumSize ( int rows) {
	return getMinimumSize();
}

public Dimension getPreferredSize () {
	return new Dimension( 100, 100);
}

public Dimension getPreferredSize ( int rows) {
	return getPreferredSize();
}

public int getRows () {
	return items.size();
}

public synchronized int getSelectedIndex () {
	if ( selections.size() == 0 )
		return -1;
	return items.indexOf( selections.firstElement() );
}

public synchronized int[] getSelectedIndexes () {
	int il = selections.size();
	int[] ia = new int[ il];

	for ( int i=0; i<il; i++)
		ia[i] = items.indexOf( selections.elementAt( i));

	return ia;
}

public synchronized String getSelectedItem () {
	return (selections.size() > 0 ) ? (String)selections.firstElement() : null;
}

public synchronized String[] getSelectedItems () {
	int il = selections.size();
	String[] sta = new String[ il];

	for ( int i=0; i<il; i++)
		sta[i] = (String)selections.elementAt( i);

	return sta;
}

public Object[] getSelectedObjects () {
	int il = selections.size();
	Object[] oa = new Object[ il];

	for ( int i=0; i<il; i++)
		oa[i] = selections.elementAt( i);

	return oa;
}

public int getVisibleIndex () {
	return 0;
}


public boolean isIndexSelected ( int index) {
	Object o = items.elementAt( index);
	return selections.contains( o);
}

public boolean isMultipleMode () {
	return multipleMode;
}

void notifyAction () {
	String s = selections.size() > 0 ? (String)selections.firstElement() : null;
	if ( ( s != null) && (hasToNotify( AWTEvent.ACTION_EVENT_MASK, aListener)) ){
	  ActionEvent ae = AWTEvent.getActionEvent( this, ActionEvent.ACTION_PERFORMED);
	  ae.setActionEvent( s, 0);
	  Toolkit.eventQueue.postEvent( ae);
	}
}

protected String paramString() {
	return super.paramString();
}

protected void processActionEvent( ActionEvent e) {
  if ( aListener != null )
	aListener.actionPerformed( e);
}

protected void processKeyEvent( KeyEvent e) {
    if ( (e.getID() == KeyEvent.KEY_PRESSED) && (e.getKeyCode() == KeyEvent.VK_ENTER) )
      notifyAction();
    super.processKeyEvent( e);
}

protected void processMouseEvent( MouseEvent e) {
    if ( e.getID() == MouseEvent.MOUSE_CLICKED )
      notifyAction();
    super.processMouseEvent( e);
}

protected void processItemEvent( ItemEvent e) {

  int idx = ((Integer)e.getItem()).intValue();
  Object o = items.elementAt( idx);

  switch ( e.getID() ) {
  case ItemEvent.SELECTED:
    if ( ! selections.contains( o) )
      selections.addElement( o);
    break;
  case ItemEvent.DESELECTED:
    if ( selections.contains( o) )
      selections.removeElement( o);
    break;
  }

  if ( iListener != null )
    iListener.itemStateChanged( e);

}

public synchronized void remove ( String item) {
	removeElement( items.indexOf( item));
}

public synchronized void remove ( int index) {
	removeElement( index);
}

public synchronized void removeActionListener ( ActionListener l) {
	aListener = AWTEventMulticaster.remove( aListener, l);
}

public synchronized void removeAll () {
      items.removeAllElements();
      if ( nativeData != null )
	Toolkit.lstRemoveAll( nativeData);

}

  void replaceSel( int idx, String newObj ) {
    Object o = items.elementAt( idx);
    if ( selections.contains( o) ) {
      if ( newObj == null )
	selections.removeElement( o);
      else
	selections.setElementAt( newObj, selections.indexOf( o));
    }
      
  }
void removeElement ( int index) {
      replaceSel( index, null) ;
      items.removeElementAt( index);
      if ( nativeData != null )
	Toolkit.lstRemoveItem( nativeData, index);

}


public synchronized void removeItemListener ( ItemListener l) {
	iListener = AWTEventMulticaster.remove( iListener, l);
}


public synchronized void replaceItem ( String newValue, int index) {
      replaceSel( index, newValue);
      items.setElementAt( newValue, index);
      if ( nativeData != null )
	Toolkit.lstReplaceItem( nativeData, newValue, index);

}


public void select ( int index) {
	if ( (index < 0) || (index > items.size() - 1) )
		return;

	if ( nativeData != null )
	  Toolkit.lstSelectItem( nativeData, index);
}


public synchronized void setMultipleMode ( boolean b) {
	multipleMode = b;
	if ( nativeData != null )
	  Toolkit.lstSetMultipleMode( nativeData, b);
}

}
