/**
 * Component - abstract root of all widgets
 *
 * Copyright (c) 1998
 *    Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 *
 */

package java.awt;

import kaffe.util.Ptr;
import java.awt.event.PaintEvent;
import java.awt.peer.ComponentPeer;

abstract public class NativeComponent
  extends Component

{
Ptr nativeData;

public void addNotify () {
  if ( nativeData != null ) {
    Toolkit.wdgSetBounds( nativeData, x, y, width, height);
    Toolkit.wdgSetBackground( nativeData, getBackground().nativeValue);
    Toolkit.wdgSetForeground( nativeData, getForeground().nativeValue);
    Toolkit.wdgSetFont( nativeData, getFont().nativeData);
    AWTEvent.registerSource( this, nativeData);
    super.addNotify();

    Toolkit.wndAddWidget( ((Window)parent).nativeData, nativeData);
   }
}


ComponentPeer createPeer () {
	return Toolkit.singleton.createLightweight( this);
}


public void paint ( Graphics g ) {
}


void paintBorder ( Graphics g, int left, int top, int right, int bottom ) {
}


void processPaintEvent ( PaintEvent e ) {
}


public void removeNotify () {
  if ( nativeData != null ) {
	AWTEvent.unregisterSource( this, nativeData);
	nativeData = null;
	super.removeNotify();
  }
}


public void repaint ( int x, int y, int width, int height ) {
}

public void repaint ( long ms ) {
}

public void repaint ( long ms, int x, int y, int width, int height ) {
}

public void requestFocus () {
}

public void setBackground ( Color clr ) {
	bgClr = clr;
	
	if ( nativeData != null )
	  Toolkit.wdgSetBackground( nativeData, bgClr.nativeValue);
}

public void setBounds ( int xNew, int yNew, int wNew, int hNew ) {
        super.setBounds( xNew, yNew, wNew, hNew);

	if ( nativeData != null )
	  Toolkit.wdgSetBounds( nativeData, x, y, width, height);

}

public void setEnabled ( boolean isEnabled ) {
  super.setEnabled( isEnabled);
}

public void setFont ( Font newFont ) {
	font = newFont;

	if ( nativeData != null )
	  Toolkit.wdgSetFont( nativeData, font.nativeData);

}

public void setForeground ( Color clr ) {
	fgClr = clr;
	
	if ( nativeData != null )
	  Toolkit.wdgSetForeground( nativeData, fgClr.nativeValue);
}


void setNativeCursor ( Cursor cursor ) {
  //  if ( nativeData != null )
  //    Toolkit.wdgSetCursor( nativeData, cursor.type);
}


public void setVisible ( boolean showIt ) {
	if ( showIt != isVisible ) {
	  super.setVisible( showIt);
	  if ( nativeData != null )
	     Toolkit.wdgSetVisible( nativeData, showIt);
	}
}


PopupMenu triggerPopup ( int idx, int x, int y ) {
  /*
	if ( popups != null && (popups.size() > idx) ){
		PopupMenu p = (PopupMenu)popups.elementAt( idx);
		p.show( this, x, y);
		return p;
	}
  */
	return null;
}


}




