package java.awt;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import kaffe.util.Ptr;

/**
 * class Button - 
 *
 * Copyright (c) 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */
public class Button
  extends NativeComponent
{
  String label;
  ActionListener aListener;
  String aCmd;

public Button () {
	this( "");
}

public Button ( String label) {
	cursor = Cursor.getPredefinedCursor( Cursor.HAND_CURSOR);
	
	setFont( Defaults.BtnFont);
	setBackground( Defaults.BtnClr);
	setForeground( Defaults.BtnTxtClr);
	setLabel( label);
}

public void addNotify() {
  if ( nativeData == null ) {
      nativeData = Toolkit.btnCreateButton( label);
      super.addNotify();
  }			
}

public void addActionListener ( ActionListener a) {
	aListener = AWTEventMulticaster.add( aListener, a);
	eventMask |= AWTEvent.ACTION_EVENT_MASK;
}

public String getActionCommand () {
	return ( (aCmd != null) ? aCmd : label);
}

public String getLabel() {
	return label;
}

public Dimension getMinimumSize() {
    return getPreferredSize();
}

public Dimension getPreferredSize () {
	int cx = 40;
	int cy = 20;
	FontMetrics fm = getFontMetrics( getFont() );
	int ax = fm.stringWidth( label) + 10;
	int ay = 3*fm.getHeight()/2;
	
	if ( cx < ax )
	  cx = ax;
	if ( cy < ay )
	  cy = ay;

	return new Dimension( cx, cy);
}


void notifyAction() {
	if ( hasToNotify( AWTEvent.ACTION_EVENT_MASK, aListener)) {
		ActionEvent ae = AWTEvent.getActionEvent( this, ActionEvent.ACTION_PERFORMED);
		ae.setActionEvent( getActionCommand(), 0 );
		Toolkit.eventQueue.postEvent( ae);
	}
}


protected String paramString() {
	return (super.paramString() + ",Label: " + label);
}

protected void processMouseEvent( MouseEvent e) {
  if ( e.getID() == e.MOUSE_CLICKED )
    notifyAction();
  super.processMouseEvent( e);
}

protected void processActionEvent( ActionEvent e) {
	aListener.actionPerformed( e);
}

public void removeActionListener ( ActionListener a) {
	aListener = AWTEventMulticaster.remove( aListener, a);
}

public void setActionCommand ( String aCmd) {
	this.aCmd = aCmd;
}

public void setLabel ( String label) {
	this.label = label;

	if ( nativeData != null )
	  Toolkit.btnSetLabel( nativeData, label);
}
}



