/*
 * Copyright (c) 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */


package java.awt;


import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

public class Checkbox
extends NativeComponent
implements ItemSelectable
{
  ItemListener iListener;
  CheckboxGroup group;
  boolean state;
  String label;

  public Checkbox() {
    this( "", false, null);
  }

  public Checkbox( String label) {
    this( label, false, null);
  }

  public Checkbox ( String label, boolean state) {
    this( label, state, null);
  }

  public Checkbox( String label, boolean state, CheckboxGroup group) {
    cursor = Cursor.getPredefinedCursor( Cursor.HAND_CURSOR);
 
    this.label = label;
    this.state = state;
    setCheckboxGroup( group);
  }

  public void addNotify() {
    if ( nativeData == null ) {
      nativeData = Toolkit.cbxCreateCheckbox( label);
      setState( state);
      super.addNotify();
    }
  }

  public String getLabel() {
    return label;
  }

  public void setLabel( String label) {
    this.label = label;
    if ( nativeData != null )
      Toolkit.cbxSetLabel( nativeData, label);
  }

  public boolean getState() {
    if ( nativeData != null )
      state = Toolkit.cbxGetState( nativeData);
    return state;
  }

  public void setState( boolean state) {
    if ( this.state == state )
      return;

    if ( state && (group != null ))
      group.setSelectedCheckbox( this);
    else
      this.state = state;

    if ( nativeData != null )
      Toolkit.cbxSetState( nativeData, state);
  }

  public Object[] getSelectedObjects() {
    Object[] so = { null };
    if ( state )
      so[0] = this;
    return so;
  }

  public CheckboxGroup getCheckboxGroup() {
    return group;
  }

  public void setCheckboxGroup( CheckboxGroup group) {
    if ( this.group == group )
      return;

    if ( this.group != null )
      this.group.boxes.removeElement( this);

    this.group = group;

    if ( this.group != null )
      this.group.boxes.addElement( this);
      
  }

  public void addItemListener( ItemListener l) {
    iListener = AWTEventMulticaster.add( iListener, l);
  }

  public void removeItemListener( ItemListener l) {
    iListener = AWTEventMulticaster.remove( iListener, l);
  }

  protected void processItemEvent( ItemEvent e) {

    setState ( e.getID() == ItemEvent.SELECTED );

    if ( iListener != null ){
      iListener.itemStateChanged( e);
    }
  }

  protected String paramString() {
    return "";
  }
}


