/*
 * Copyright (c) 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */


package java.awt;


import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;

public class Choice
extends NativeComponent
implements ItemSelectable
{
  ItemListener iListener;
  Vector items = new Vector();
  int selection = -1;

  public Choice() {
  }

  public void addNotify() {
    if ( nativeData == null ) {
      nativeData = Toolkit.choCreateChoice();
      updateItems();
      super.addNotify();
    }
  }

  public int getItemCount() {
    return items.size();
  }

  public String getItem( int idx) {
    return (String)items.elementAt( idx);
  }

  public void add( String item) {
    insert( item, -1);
  }

  public void addItem( String item) {
    insert( item, -1);
  }

  public void insert( String item, int idx) {
    if ( idx == -1 ) {
      items.addElement( item);
    }
    else {
      items.insertElementAt( item, idx);
    }
    updateItems();
  }

  public void remove( String item) {
    remove( items.indexOf( item));
  }

  public void remove( int idx) {
    items.removeElementAt( idx);
    if ( selection == idx )
      selection = -1;
    updateItems();
  }

  public void removeAll() {
    items.removeAllElements();
    selection = -1;
    updateItems();
  }

  public String getSelectedItem() {
    if ( selection == -1 )
      return null;
    return (String)items.elementAt( selection);
  }

  public Object[] getSelectedObjects() {
    Object[] so = { null };
    if ( selection > -1 )
      so[0] = items.elementAt( selection);
    return so;
  }

  public int getSelectedIndex() {
    return selection;
  }

  public void select( int idx) {
    selection = idx;
    if ( nativeData != null )
      Toolkit.choSelectItem( nativeData, idx);
  }

  public void select( String item) {
    select( items.indexOf( item));
  }

  public void addItemListener( ItemListener l) {
    iListener = AWTEventMulticaster.add( iListener, l);
  }

  public void removeItemListener( ItemListener l) {
    iListener = AWTEventMulticaster.remove( iListener, l);
  }

  protected void processItemEvent( ItemEvent e) {
    int idx = ((Integer)e.getItem()).intValue();
    Object o = items.elementAt( idx);

    switch ( e.getID() ) {
    case ItemEvent.SELECTED:
      if ( selection != idx ) {
	selection = idx;
	if ( iListener != null )
	  iListener.itemStateChanged( e);
      }
      break;
    }
  }

  protected String paramString() {
    return "";
  }

  void updateItems() {
    if ( nativeData != null ) {
      Toolkit.choRemoveAll( nativeData);
      for ( int i=0; i<items.size(); i++)
	Toolkit.choAppendItem( nativeData, (String)items.elementAt( i));
    }
  }

}
