/*
 * Copyright (c) 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution 
 * of this file. 
 */


package java.awt;

public class TextArea
extends TextComponent
{

  int rows;
  int cols;
  public static final int SCROLLBARS_BOTH = 0;
  public static final int SCROLLBARS_VERTICAL_ONLY = 1;
  public static final int SCROLLBARS_HORIZONTAL_ONLY = 2;
  public static final int SCROLLBARS_NONE = 3;
  int scrolls;

  public TextArea() {
    this( "", 0, 0, SCROLLBARS_BOTH);
  }

  public TextArea( String text) {
    this( text, 0, 0, SCROLLBARS_BOTH);
  }

  public TextArea( int rows, int cols) {
    this( "", rows, cols, SCROLLBARS_BOTH);
  }

  public TextArea( String text, int rows, int cols) {
    this( text, rows, cols, SCROLLBARS_BOTH);
  }

  public TextArea( String text, int rows, int cols, int scrolls) {
    this.rows = rows;
    this.cols = cols;
    this.scrolls = scrolls;

    cursor = Cursor.getPredefinedCursor( Cursor.TEXT_CURSOR);
    setForeground( Defaults.TextAreaTxtClr);
    setBackground( Defaults.TextAreaBgClr);
    setFont( Defaults.TextAreaFont);
    setText( text);
  }

  public void addNotify() {
    if ( nativeData == null ) {
      nativeData = Toolkit.txtCreateText( text);
      super.addNotify();
    }
  }

  public void insert( String text, int pos) {
    if ( nativeData != null )
      Toolkit.txtInsert( nativeData, text, pos);
  }

  public void append( String text) {
    if ( nativeData != null )
      Toolkit.txtAppend( nativeData, text);
  }

  public void replaceRange( String text, int start, int end) {
    if ( nativeData != null )
      Toolkit.txtReplaceRange( nativeData, text, start, end);
  }

  public int getRows() {
    return rows;
  }

  public void setRows( int rows) {
    this.rows = rows;
  }

  public int getColumns() {
    return cols;
  }

  public void setColumns( int cols) {
    this.cols = cols;
  }

  public int getScrollbarVisibility() {
    return scrolls;
  }

  public Dimension getPreferredSize( int rows, int cols) {
    FontMetrics fm = getFontMetrics( getFont() );
    return new Dimension( cols * fm.charWidth('x'), rows * fm.getHeight() );
  }

  public Dimension getPreferredSize() {
    return getPreferredSize( 10, 20);
  }

  public Dimension getMinimumSize( int cols, int rows) {
    return new Dimension( 50, 50);
  }

  public Dimension getMinimumSize() {
    return new Dimension( 50, 50);
  }

  protected String paramString() {
    return "";
  }
}

